@@ -4,7 +4,10 @@ use strict;
 use warnings;
 
 my @TEST_PCS = (
-    { raw => 'X', partial => 1 },
+    {   raw     => 'X',
+        partial => 1,
+        lax     => 0,
+    },
     {   raw          => 'XX1',
         area         => 'XX',
         district     => '1',
@@ -41,7 +44,9 @@ my @TEST_PCS = (
         partial      => 1,
         fixed_format => 'XX1X 1  ',
     },
-    { raw => 'XX11X1XX' },
+    {   raw => 'XX11X1XX',
+        lax => 0,
+    },
     {   raw          => 'QI1 1AA',
         area         => 'QI',
         district     => '1',
@@ -50,13 +55,23 @@ my @TEST_PCS = (
         unit         => 'AA',
         fixed_format => 'QI1  1AA',
     },
-    {   raw           => 'AB10 1II',
-        area          => 'AB',
-        district      => '10',
-        subdistrict   => undef,
-        sector        => '1',
-        unit          => 'II',
-        fixed_format  => 'AB10 1II',
+    {   raw          => 'AB10 1II',
+        area         => 'AB',
+        district     => '10',
+        subdistrict  => undef,
+        sector       => '1',
+        unit         => 'II',
+        fixed_format => 'AB10 1II',
+        posttowns    => ['ABERDEEN'],
+        unanchored   => {
+            sector         => undef,
+            unit           => undef,
+            strict         => 1,
+            valid          => 1,
+            captures       => [ 'AB', '10' ],
+            valid_captures => ['AB10'],
+            needs_space    => 1,
+        },
     },
     {   raw          => 'A1 1AA',
         area         => 'A',
@@ -121,6 +136,7 @@ my @TEST_PCS = (
         strict       => 1,
         valid        => 1,
         fixed_format => 'AB10 1AA',
+        posttowns    => ['ABERDEEN'],
     },
     {   raw              => 'AB99 1AA',
         area             => 'AB',
@@ -132,6 +148,7 @@ my @TEST_PCS = (
         valid            => 1,
         non_geographical => 1,
         fixed_format     => 'AB99 1AA',
+        posttowns        => ['ABERDEEN'],
     },
     {   raw              => 'BX99 1AA',
         area             => 'BX',
@@ -154,6 +171,7 @@ my @TEST_PCS = (
         strict       => 1,
         valid        => 1,
         fixed_format => 'SE1     ',
+        posttowns    => ['LONDON'],
     },
     {   raw              => 'BF1 1AA',
         area             => 'BF',
@@ -167,6 +185,7 @@ my @TEST_PCS = (
         non_geographical => 1,
         bfpo             => 1,
         fixed_format     => 'BF1  1AA',
+        posttowns        => ['BFPO'],
     },
     {   raw          => 'AB1 2CD',
         area         => 'AB',
@@ -177,6 +196,17 @@ my @TEST_PCS = (
         fixed_format => 'AB1  2CD',
         valid        => 0,
         strict       => 0,
+        posttowns    => [],
+        unanchored   => {
+            sector         => undef,
+            unit           => undef,
+            fixed_format   => 'AB1',
+            partial        => 1,
+            strict         => 1,
+            captures       => [ 'AB', '1' ],
+            valid_captures => ['AB1'],
+            needs_space    => 1,
+        },
     },
     {   raw          => 'WC1H 9EB',
         area         => 'WC',
@@ -187,6 +217,7 @@ my @TEST_PCS = (
         fixed_format => 'WC1H 9EB',
         valid        => 1,
         strict       => 1,
+        posttowns    => ['LONDON'],
     },
     {   raw          => 'AB1',
         area         => 'AB',
@@ -198,6 +229,7 @@ my @TEST_PCS = (
         valid        => 0,
         partial      => 1,
         strict       => 1,
+        posttowns    => [],
     },
     {   raw          => 'SE1',
         area         => 'SE',
@@ -209,6 +241,7 @@ my @TEST_PCS = (
         valid        => 1,
         partial      => 1,
         strict       => 1,
+        posttowns    => ['LONDON'],
     },
     {   raw          => 'SE1 0LH',
         area         => 'SE',
@@ -220,6 +253,7 @@ my @TEST_PCS = (
         valid        => 1,
         partial      => 0,
         strict       => 1,
+        posttowns    => ['LONDON'],
     },
     {   raw          => 'WC1H 9',
         area         => 'WC',
@@ -231,6 +265,7 @@ my @TEST_PCS = (
         valid        => 1,
         partial      => 1,
         strict       => 1,
+        posttowns    => ['LONDON'],
     },
     {   raw              => 'AB99 1AA',
         area             => 'AB',
@@ -243,6 +278,7 @@ my @TEST_PCS = (
         partial          => 0,
         strict           => 1,
         non_geographical => 1,
+        posttowns        => ['ABERDEEN'],
     },
     {   raw              => 'BF1 1AA',
         area             => 'BF',
@@ -256,6 +292,7 @@ my @TEST_PCS = (
         strict           => 1,
         non_geographical => 1,
         bfpo             => 1,
+        posttowns        => ['BFPO'],
     },
     {   raw          => 'B11',
         area         => 'B',
@@ -267,6 +304,7 @@ my @TEST_PCS = (
         valid        => 1,
         partial      => 1,
         strict       => 1,
+        posttowns    => ['BIRMINGHAM'],
     },
     {   raw          => 'B1 1',
         needs_space  => 1,
@@ -279,8 +317,8 @@ my @TEST_PCS = (
         valid        => 1,
         partial      => 1,
         strict       => 1,
+        posttowns    => ['BIRMINGHAM'],
     },
-
     {   raw          => 'N1 0XX',
         area         => 'N',
         district     => '1',
@@ -290,6 +328,7 @@ my @TEST_PCS = (
         fixed_format => 'N1   0XX',
         valid        => 1,
         strict       => 1,
+        posttowns    => ['LONDON'],
     },
     {   raw              => 'N1P 2NG',
         area             => 'N',
@@ -301,6 +340,7 @@ my @TEST_PCS = (
         valid            => 1,
         strict           => 1,
         non_geographical => 1,
+        posttowns        => ['LONDON'],
     },
     {   raw              => 'E2 0HP',
         area             => 'E',
@@ -312,9 +352,16 @@ my @TEST_PCS = (
         valid            => 1,
         strict           => 1,
         non_geographical => 0,
+        posttowns        => ['LONDON'],
     },
 );
 
+sub test_pcs_raw {
+    my $class = shift;
+
+    return map { $_->{raw} } @TEST_PCS;
+}
+
 sub test_pcs {
     my ( $class, $args ) = @_;
 
@@ -331,6 +378,8 @@ sub test_pcs {
     foreach my $pc (@pcs) {
         next unless $pc->{area};
 
+        $pc->{lax} //= 1;
+
         my ( $area, $district, $subdistrict, $sector, $unit )
             = map { $pc->{$_} } qw/ area district subdistrict sector unit /;
 
@@ -350,7 +399,6 @@ sub test_pcs {
         ];
 
         $pc->{valid_captures} = [ $pc->{outcode}, $sector, $unit ];
-
     }
 
     return @pcs;
